﻿Omniscience = LibStub("AceAddon-3.0"):NewAddon("Omniscience", "AceEvent-3.0", "LibBars-1.0")
local bars = LibStub("LibBars-1.0")
local media = LibStub("LibSharedMedia-3.0")
local bitband = bit.band
local floor = math.floor

local defaults = {
	profile = {
	texture = "Ruben",
	font = "ABF",
	fontsize = 9,
	playeronly = true,
	width = "250",
	height = "12",
	anchors = { 
		cd =   {name = "Cooldowns",     enabled = true, position = {}},
		cast = {name = "Casts",         enabled = true, position = {}},
		buff = {name = "Buffs",         enabled = true, position = {}},
		cc =   {name = "Crowd Control", enabled = true, position = {}},
	},
	map = {
		cd = "cd",
		cast = "cast",
		buff = "buff",
		cc = "cc",
	},
    },
}

function Omniscience:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("OmniscienceDB", defaults, "Default")
	for k in pairs(self.db.profile.anchors) do
		local anchor = k
		self[anchor] = self:NewBarGroup(self.db.profile.anchors[anchor].name, nil, self.db.profile.width, self.db.profile.height)
		self[anchor]:SetTexture(media:Fetch("statusbar", self.db.profile.texture))
		self[anchor]:SetFont(media:Fetch("font", self.db.profile.font, self.db.profile.fontsize))
		self[anchor]:SetFlashPeriod(0)
		self[anchor]:SetColorAt(0.00, 1, 0, 0, 1)
		self[anchor]:SetColorAt(0.33, 1, 1, 0, 1)
		self[anchor]:SetColorAt(0.66, 0, 1, 1, 1)
		self[anchor]:SetColorAt(1.00, 0, 0, 1, 1)
		self[anchor]:ClearAllPoints()
		if not self.db.profile.anchors[anchor].position then self.db.profile.anchors[anchor].position = {} end
		if self.db.profile.anchors[anchor].position.p
			and self.db.profile.anchors[anchor].position.pr
			and self.db.profile.anchors[anchor].position.px
			and self.db.profile.anchors[anchor].position.py then
				self[anchor]:ClearAllPoints()
				self[anchor]:SetPoint(self.db.profile.anchors[anchor].position.p, UIParent, self.db.profile.anchors[anchor].position.pr, self.db.profile.anchors[anchor].position.px, self.db.profile.anchors[anchor].position.py)
		else
			self[anchor]:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
		end
		self[anchor]:SetScale(0.75)
		self[anchor].id = anchor
		self[anchor].RegisterCallback(self, "AnchorClicked")
		self[anchor]:HideAnchor()
	end
end

function Omniscience:UpdateMedia()
	for k in pairs(self.db.profile.anchors) do
		self[k]:SetTexture(media:Fetch("statusbar", self.db.profile.texture))
		self[k]:SetFont(media:Fetch("font", self.db.profile.font, self.db.profile.fontsize))

	end
end

function Omniscience:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	media.RegisterCallback(self, "LibSharedMedia_Registered", "UpdateMedia")  
	media.RegisterCallback(self, "LibSharedMedia_SetGlobal", "UpdateMedia")  
	self:UpdateMedia()
end

function Omniscience:OnDisable()
	self:UnRegisterAllEvents()
	media.UnregisterCallback(self)
	for k in pairs(self.db.profile.anchors) do
		local anchor = k
		self[anchor].UnregisterAllCallbacks()
	end
end

function Omniscience:AnchorClicked(callback, group, button)
	if button == "RightButton" then
		local anchor = group.id
		self:SavePosition(anchor)
		self[anchor]:HideAnchor()
	end
end

function Omniscience:ToggleAnchors()
	for k in pairs(self.db.profile.anchors) do
		local anchor = k
		local p,_,pr,px,py  = self[anchor]:GetPoint()
		self:SavePosition(anchor)
		self[anchor]:ToggleAnchor()
	end
end

function Omniscience:SavePosition(anchor)
	local p,_,pr,px,py = self[anchor]:GetPoint()
	self.db.profile.anchors[anchor].position.p = p
	self.db.profile.anchors[anchor].position.pr = pr
	self.db.profile.anchors[anchor].position.px = px
	self.db.profile.anchors[anchor].position.py = py
end

function Omniscience:RunTest()
	for k in pairs(self.db.profile.anchors) do
		local anchor = k
		local icon = select(3, GetSpellInfo(43308))
		self[anchor]:NewTimerBar("test-"..anchor, self.db.profile.anchors[anchor].name, 30, nil, icon)
	end
end

function Omniscience:CheckAffiliation(flags, match)
	local result = (bitband(flags, match) ~= 0)
	return result or nil
end

function Omniscience:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellID, spellName, spellFlags, ...)
	if eventType == "SPELL_INTERRUPT" then
		if not self.spells["SPELL_CAST_START"][spellID] then return end
		local group = self.spells["SPELL_CAST_START"][spellID].group
		local anchor = self.db.profile.map[group]
		local interrupted = select(1, ...)
		local name = "Omniscience-"..dstGUID.."-"..interrupted
		local bar = self[anchor]:GetBar(name)
		if bar then bar:ReleaseBar() end
	return end

	if eventType == "SPELL_AURA_REMOVED" then
		if not self.spells["SPELL_AURA_APPLIED"][spellID] then return end
		local group = self.spells["SPELL_AURA_APPLIED"][spellID].group
		local anchor = self.db.profile.map[group]
		local name = "Omniscience-"..dstGUID.."-"..spellID
		local bar = self[anchor]:GetBar(name)
		if bar then bar:ReleaseBar() end
	return end

	if not self.spells[eventType] then return end
	if not self.spells[eventType][spellID] then return end
	local flags, match, actor, actorGUID
	if self.spells[eventType][spellID].srcflags then
		flags, match = srcFlags, self.spells[eventType][spellID].srcflags
		actor, actorGUID = srcName, srcGUID
	elseif self.spells[eventType][spellID].dstflags then
		flags, match = dstFlags, self.spells[eventType][spellID].dstflags
		actor, actorGUID = dstName, dstGUID
	end
	if not actor or not actorGUID or not flags or not match then return end
	if not self:CheckAffiliation(flags, match) then return end

	local name = "Omniscience-"..actorGUID.."-"..spellID
	local value = self.spells[eventType][spellID].time
	local text = actor..": "..spellName
	local icon = select(3, GetSpellInfo(spellID)) or nil
	local group = self.spells[eventType][spellID].group
	local anchor = self.db.profile.map[group]
	self[anchor]:NewTimerBar(name, text, value, nil, icon)
end
